/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/


#ifdef HAS_GLOBUS_GSS

#include "auth.h"
#include "debug.h"
#include "globus_gss_assist.h"

#include <stdio.h>
#include <unistd.h>

static int auth_globus_barrier( FILE *r, FILE *w, int value )
{
	int fields, response;
	char line[AUTH_LINE_MAX];

	fprintf(w,"%d\n",value);
	fflush(w);

	if(fgets(line,AUTH_LINE_MAX,r)) {
		fields = sscanf(line,"%d",&response);
		if(fields==1) {
			return response;
		}
	}

	return 0;
}

static int auth_globus_assert( FILE *r, FILE *w )
{
	gss_cred_id_t credential = GSS_C_NO_CREDENTIAL;
	gss_ctx_id_t context = GSS_C_NO_CONTEXT;
	OM_uint32 major, minor, flags=0;
	int token;
	int result;

	major = globus_gss_assist_acquire_cred(&minor,GSS_C_INITIATE,&credential); 
	if( major!=GSS_S_COMPLETE ) {
		result = 0;
	} else {
		result = 1;
	}

	if(!auth_globus_barrier(r,w,result)) {
		debug("auth_globus: other side couldn't load credentials\n");
		result = 0;
		goto done;
	}

	if(!result) {
		debug("auth_globus: couldn't load my credentials: did you run grid-proxy-init?\n");
		goto done;
	}

	debug("auth_globus: sending credentials\n");

	setbuf(w,0);

	major = globus_gss_assist_init_sec_context( &minor, credential, &context, 0, 0, &flags, &token, globus_gss_assist_token_get_fd, r, globus_gss_assist_token_send_fd, w );
	if( major!=GSS_S_COMPLETE ) {
		debug("auth_globus: couldn't authenticate\n");
		result = 0;
		goto done;
	}

	debug("auth_globus: credentials accepted\n");
	result = 1;

	done:

	setvbuf(w,0,_IOFBF,BUFSIZ);
	gss_delete_sec_context( &minor, &context, GSS_C_NO_BUFFER );
	gss_release_cred( &major, &credential );

	return result;
}

static int auth_globus_accept( FILE *r, FILE *w, char **subject )
{
	gss_cred_id_t credential = GSS_C_NO_CREDENTIAL;
	gss_ctx_id_t context = GSS_C_NO_CONTEXT;
	OM_uint32 major, minor, flags=0;
	int token;
	int result;

	*subject = 0;
	major = globus_gss_assist_acquire_cred(&minor,GSS_C_ACCEPT,&credential); 
	if( major!=GSS_S_COMPLETE ) {
		result = 0;
	} else {
		result = 1;
	}

	if(!auth_globus_barrier(r,w,result)) {
		debug("auth_globus: other side couldn't load credentials\n");
		result = 0;
		goto done;
	}

	if(!result) {
		debug("auth_globus: couldn't load my credentials: did you run grid-proxy-init?\n");
		goto done;
	}

	debug("auth_globus: receiving credentials\n");

	setbuf(w,0);

	major = globus_gss_assist_accept_sec_context( &minor, &context, credential, subject, &flags, 0, &token, 0, globus_gss_assist_token_get_fd, r, globus_gss_assist_token_send_fd, w );
	if( major!=GSS_S_COMPLETE ) {
		debug("auth_globus: couldn't authenticate\n");
		*subject = 0;
		result = 0;
		goto done;
	}

	debug("auth_globus: remote user is '%s'\n",*subject);
	result = 1;

	done:
	setvbuf(w,0,_IOFBF,BUFSIZ);
	gss_delete_sec_context( &minor, &context, GSS_C_NO_BUFFER );
	gss_release_cred( &major, &credential );

	return result;
}

int auth_globus_register()
{
	return auth_register( "globus", auth_globus_assert, auth_globus_accept );
}

#endif
