/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/


#include "client_cache.h"
#include "kangaroo_client.h"
#include "network.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>

struct client_cache {
	struct kangaroo_server *kserver;
	char *kserver_name;
	int kserver_port;

	int file;
	char *file_name;
	int file_flags;

	void *buffer;
	int buffer_size;
};

struct client_cache * client_cache_create()
{
	struct client_cache *c;

	c = malloc(sizeof(*c));
	if(!c) return 0;

	c->kserver=0;
	c->kserver_name=0;
	c->kserver_port=0;

	c->file=-1;
	c->file_name = 0;
	c->file_flags = 0;

	c->buffer=0;
	c->buffer_size=0;

	return c;
}

void client_cache_delete( struct client_cache *c )
{
	if(c->kserver) kangaroo_server_close(c->kserver);
	if(c->file>=0) close(c->file);
	if(c->buffer) free(c->buffer);
}

int client_cache_open( struct client_cache *c, const char *path, int flags, int mode )
{
	if( c->file>=0 && !strcmp(c->file_name,path) && flags==c->file_flags ) {
		return c->file;
	}

	if( c->file_name ) {
		close(c->file);
		c->file = -1;
		free(c->file_name);
		c->file_name = 0;
	}

	c->file = open(path,flags,mode);
	if(c->file<0) return c->file;

	c->file_name = strdup(path);
	if(!c->file_name) {
		close(c->file);
		return -1;
	}

	c->file_flags = flags;

	return c->file;
}

void * client_cache_malloc( struct client_cache *c, int bytes )
{
	if( c->buffer && bytes<c->buffer_size ) {
		return c->buffer;
	}

	if( c->buffer ) {
		free(c->buffer);
		c->buffer = 0;
	}

	c->buffer_size = bytes;
	return (c->buffer=malloc(bytes));
}

struct kangaroo_server * client_cache_kangaroo_server_open( struct client_cache *c, const char *host, int port )
{
	if( c->kserver && !strcmp(c->kserver_name,host) && (c->kserver_port==port) ) {
		return c->kserver;
	}

	if( c->kserver ) {
		kangaroo_server_close(c->kserver);
		free(c->kserver_name);
		c->kserver = 0;
		c->kserver_name = 0;
		c->kserver_port = 0;
	}

	c->kserver_name = strdup(host);
	if(!c->kserver_name) return 0;

	c->kserver = kangaroo_server_open(host,port);
	if(!c->kserver) {
		free(c->kserver_name);
		return 0;
	}

	c->kserver_port = port;

	return c->kserver;
}

void client_cache_kangaroo_server_close( struct client_cache *c )
{
	if(c->kserver) {
		kangaroo_server_close(c->kserver);
		c->kserver = 0;
		free(c->kserver_name);
		c->kserver_name = 0;
	}
}

