
/*
 Part of multi route version of kangaroo
 Written by Gogul Balakrishnan and Rajesh Rajamani
*/ 

#ifndef __DEP_GRAPH_H
#define __DEP_GRAPH_H

#include <pthread.h>
#include "kangaroo_message.h"

#define DEP_GRAPH_TABLE_BUCKET_COUNT 10
#define DEP_GRAPH_MAX_MOVERS 5

struct dep_graph_node{
  struct kangaroo_message *m;
  int dependent_on_count;
  struct list *dependents;
};

struct dep_graph {
  struct list *node_list;
  struct list *parallel_ops;
  struct list *waiting_for_push[DEP_GRAPH_MAX_MOVERS];
	int resend_flags[DEP_GRAPH_MAX_MOVERS];
	struct dep_graph_node *resend_head[DEP_GRAPH_MAX_MOVERS];
  struct dep_graph *hold_dg;
	struct dep_graph *rename_dependent;
	int waiting_for_notification;
  int hold;
  int has_hold_dg;
  int has_mover;
  char addr[KANGAROO_ADDR_MAX];
  char path[KANGAROO_PATH_MAX];
	pthread_cond_t pushcond;
  pthread_mutex_t mutex;
};


extern struct hash_table *dep_graph_table;

/*create and return a pointer to a dep_graph_node structure */
struct dep_graph_node * dep_graph_node_create();

int dep_graph_table_create();


/*create and return pointer to a dep_graph structure */
struct dep_graph * dep_graph_create(char *addr,char *path);

int dep_graph_add_dependency(void *,const void* );

struct dep_graph *dep_graph_insert_message(struct kangaroo_message *m);
struct dep_graph * dep_graph_insert_message_without_spool(struct kangaroo_message *m);
struct dep_graph * dep_graph_insert_message_internal1(struct dep_graph *correct_dg,struct kangaroo_message *m);
struct dep_graph_node *dep_graph_insert_message_internal2(struct dep_graph *dg, struct kangaroo_message *m);

int dep_graph_node_equality(void *,const void *);

int dep_graph_node_remove(struct dep_graph *dg, struct dep_graph_node *dg_node);

struct kangaroo_message * dep_graph_get_next(struct dep_graph *dg, int mover_id);

/*returns a list of (struct dep_graph_node *) */
struct list *dep_graph_what_to_push(struct dep_graph *dg, int mover_id);

/* Earlier items have been pushed*/
struct dep_graph  * dep_graph_pushed(struct dep_graph *dg, int mover_id);

int dep_graph_number_next_hops (struct dep_graph *dg, int number);

/* 1 is no error */
int dep_graph_rollback(struct dep_graph *dg, int mover_id, int rollback_mode);

int dep_graph_has_mover (struct dep_graph *dg);
void dep_graph_gets_mover (struct dep_graph *dg);
int dep_graph_is_empty(struct dep_graph *dg);
struct dep_graph* dep_graph_not_null(struct kangaroo_message *m);
int dep_graph_read (struct kangaroo_message *m);

char* dep_graph_destination (struct dep_graph *dg);
void dep_graph_delete(struct dep_graph *dg);
void dep_graph_push (struct kangaroo_message *m);
int dep_graph_load(void);

/* Unlike what to push, you should destruct this list 
Pass a pointer to a (struct list *) 
returning 0 means error
returning 1 means Ok 
*/
int dep_graph_give_all_dg(struct list **p_dg_list);

#endif
