/******************************************************************************
 *                                                                            *
 * File:    dllink.h                                                          *
 *                                                                            *
 * Usage:   Doublely Linked List Library Header                               *
 *                                                                            *
 * Authors: Rajesh Rajamani                                                   *
 *                                                                            *
 *****************************************************************************/
#ifndef __DLLINK_H
#define __DLLINK_H

#define DEBUG   0
#define TRUE	1
#define FALSE	0
#define COPYD	0

#include <iostream.h>
#include <stdlib.h>

typedef int (*PF) (void*,void*);
typedef void(*printfunc)(const void*);

struct DLlink
{	void *item;
	DLlink	*prev,*next;
	DLlink(void *data):item(data),prev(0),next(0) {}
	void Initialize(void *data);
};	

class DLlist
{  public:	
		void InsertFirst(DLlink*);
		void Append(DLlink*);
		DLlink* DeleteHead();
		DLlink* DeleteTail();
		DLlink* DeleteNode(void*,PF);
		void Print(printfunc) const;
		DLlist(DLlink* p=0)
		{if(p==NULL) head=tail=p; else { head=p;tail=FindTail();} }
		void Initialize(DLlink* p=0)
		{if(p==NULL) head=tail=p; else { head=p;tail=FindTail();} }
		void Cleanup();
		DLlink* FindTail();
		friend class DLlistIterator;
		DLlink* Remove(DLlink*);
		DLlink* GetHead(void) { return head; }
		DLlink* GetTail(void) { return tail; }
  private:
  		DLlink  *head,*tail;
};

class DLlistIterator
{  public:
		DLlistIterator(){IfAlloc=0;}
		DLlistIterator(DLlist& dl):pdll(&dl),index(dl.head){IfAlloc=0;prev=0;}
		DLlistIterator(DLlist* dl):pdll(dl),index(dl->head){IfAlloc=0;prev=0;}
		DLlistIterator(DLlink* link);
		void Initialize(DLlist &dl){pdll=&dl;index=dl.head;IfAlloc=0;prev=0;}
		~DLlistIterator(){if(IfAlloc) delete pdll; }
		DLlink* Next();
		void Rehash(DLlink*);
		void Reset();
  private:
		DLlist* pdll;
  		DLlink* index;
		DLlink* prev;
		int IfAlloc;
};

#endif		
