/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/


#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

/* A number of platforms fail to prototype these functions */

extern ssize_t pread( int fd, void *buf, size_t count, off_t offset );
extern ssize_t pwrite( int fd, const void *buf, size_t count, off_t offset );

ssize_t full_read( int fd, void *buf, size_t count )
{
	ssize_t total=0;
	ssize_t chunk=0;

	while( count>0 ) {
		chunk = read(fd,buf,count);
		if(chunk<0) {
			if(errno==EINTR) {
				continue;
			} else {
				break;
			}
		} else if(chunk==0) {
			break;
		} else {
			total += chunk;
			count -= chunk;
			buf += chunk;
		}
	}

	if(total>0) {
		return total;
	} else {
		if(chunk==0) {
			return 0;
		} else {
			return -1;
		}
	}		
}

ssize_t full_write( int fd, const void *buf, size_t count )
{
	ssize_t total=0;
	ssize_t chunk=0;

	while( count>0 ) {
		chunk = write(fd,buf,count);
		if(chunk<0) {
			if(errno==EINTR) {
				continue;
			} else {
				break;
			}
		} else if(chunk==0) {
			break;
		} else {
			total += chunk;
			count -= chunk;
			buf += chunk;
		}
	}

	if(total>0) {
		return total;
	} else {
		if(chunk==0) {
			return 0;
		} else {
			return -1;
		}
	}		
}

ssize_t full_pread( int fd, void *buf, size_t count, off_t offset )
{
	ssize_t total=0;
	ssize_t chunk=0;

	while( count>0 ) {
		chunk = pread(fd,buf,count,offset);
		if(chunk<0) {
			if(errno==EINTR) {
				continue;
			} else {
				break;
			}
		} else if(chunk==0) {
			break;
		} else {
			total += chunk;
			count -= chunk;
			buf += chunk;
			offset += chunk;
		}
	}

	if(total>0) {
		return total;
	} else {
		if(chunk==0) {
			return 0;
		} else {
			return -1;
		}
	}		
}

ssize_t full_pwrite( int fd, const void *buf, size_t count, off_t offset )
{
	ssize_t total=0;
	ssize_t chunk=0;

	while( count>0 ) {
		chunk = pwrite(fd,buf,count,offset);
		if(chunk<0) {
			if(errno==EINTR) {
				continue;
			} else {
				break;
			}
		} else if(chunk==0) {
			break;
		} else {
			total += chunk;
			count -= chunk;
			buf += chunk;
			offset += chunk;
		}
	}

	if(total>0) {
		return total;
	} else {
		if(chunk==0) {
			return 0;
		} else {
			return -1;
		}
	}		
}

