
#include "histogram.h"

#include <time.h>
#include <stdlib.h>

struct histogram {
	time_t first_bucket_time;
	UINT64_T *buckets;
	int bucket_count;
	int seconds_per_bucket;
};

struct histogram * histogram_create( int bucket_count, int seconds_per_bucket )
{
	struct histogram *h;

	h = malloc(sizeof(*h));
	if(!h) return 0;

	h->buckets = malloc( sizeof(*h->buckets)*bucket_count );
	if(!h->buckets) {
		free(h);
		return 0;
	}

	memset(h->buckets,0,sizeof(*h->buckets)*bucket_count);

	h->bucket_count = bucket_count;
	h->seconds_per_bucket = seconds_per_bucket;
	h->first_bucket_time = time(0);

	return h;
}

void histogram_delete( struct histogram *h )
{
	free(h->buckets);
	free(h);
}

static void histogram_shift( struct histogram *h )
{
	time_t current = time(0);
	time_t offset;
	int shift;
	int i;

	offset = current-h->first_bucket_time;
	shift = offset/h->seconds_per_bucket;

	if(shift>h->bucket_count) {
		shift = h->bucket_count;
	}

	for( i=h->bucket_count-1; i>=shift; i-- ) {
		h->buckets[i] = h->buckets[i-shift];
	}

	for( i=shift-1; i>=0; i-- ) {
		h->buckets[i] = 0;
	}

	h->first_bucket_time += h->seconds_per_bucket*shift;
}

void histogram_set( struct histogram *h, int position, UINT64_T value )
{
	histogram_shift(h);
	h->buckets[position] = value;
}

UINT64_T histogram_delta( struct histogram *h, int position, UINT64_T delta )
{
	histogram_shift(h);
	return (h->buckets[position]+=delta);
}

UINT64_T histogram_get( struct histogram *h, int position )
{
	histogram_shift(h);
	return h->buckets[position];
}

UINT64_T histogram_total( struct histogram *h )
{
	UINT64_T total=0;
	int i;

	histogram_shift(h);
	for( i=0; i<h->bucket_count; i++ ) {
		total += h->buckets[i];
	}
	return total;
}

