/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/


#ifndef KANGAROO_CLIENT_H
#define KANGAROO_CLIENT_H

#include <stdio.h>

#include "kangaroo_types.h"

int kangaroo_name_to_address( const char *name, char *addr_string );
int kangaroo_address_to_name( const char *addr_string, char *name );

struct kangaroo_server * kangaroo_server_open( const char *host, int port );
int kangaroo_server_address( struct kangaroo_server *, char *addr_string );
void kangaroo_server_close( struct kangaroo_server * );

int kangaroo_put( struct kangaroo_server *s, const char *target_host, const char *target_path, int offset, int length, const void *data );
int kangaroo_get( struct kangaroo_server *s, const char *target_host, const char *target_path, int offset, int length, void *data );

int kangaroo_create( struct kangaroo_server *s, const char *host, const char *path, int mode );
int kangaroo_delete( struct kangaroo_server *s, const char *host, const char *path );
int kangaroo_truncate( struct kangaroo_server *s, const char *host, const char *path, int length );
int kangaroo_rename( struct kangaroo_server *s, const char *host, const char *path, const char *newpath );

int kangaroo_commit( struct kangaroo_server *s );
int kangaroo_push( struct kangaroo_server *s, const char *target_host, const char *target_path );
int kangaroo_unqueue( struct kangaroo_server *s, const char *target_host, const char *target_path );
int kangaroo_route( struct kangaroo_server *s, const char *target_host, char *hop );

int kangaroo_status( struct kangaroo_server *s, char *data, int length );
int kangaroo_pause( struct kangaroo_server *s, int incoming, int outgoing );
int kangaroo_resume( struct kangaroo_server *s, int incoming, int outgoing );

struct kangaroo_message;
int kangaroo_message_send( struct kangaroo_server *s, struct kangaroo_message *m );

#endif
