/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/


#include "config_table.h"
#include "kangaroo_types.h"
#include "username.h"
#include "debug.h"

#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

struct entry {
	const char *key;
	const char *value;
};

static struct entry defaults[] = {
	{ "kangaroo.install.dir", KANGAROO_INSTALL_DIR },
	{ "kangaroo.config", "${kangaroo.install.dir}/etc/kangaroo.config" },
	{ "kangaroo.auth.file", "${kangaroo.install.dir}/etc/kangaroo.authusers" },
	{ "kangaroo.routing.file", "${kangaroo.install.dir}/etc/kangaroo.routing" },
	{ "kangaroo.spool.dir", "/tmp/kangaroo-${user}" },
	{ "kangaroo.spool.reserve", "10000000" },
	{ "kangaroo.spool.max", "500000000"},
	{ "kangaroo.local.port", "${kangaroo.spool.dir}/.port" },
	{ "kangaroo.www.port", "9096" },
	{ "kangaroo.tcp.port", "9095" },
	{ "kangaroo.mover.timeout.periodic", "300" },
	{ "kangaroo.mover.timeout.min", "5" },
	{ "kangaroo.mover.timeout.max", "60" },
	{ "kangaroo.incoming.timeout.min", "1" },
	{ "kangaroo.incoming.timeout.max", "10" },
	{ "kangaroo.notify.delay", "300" },
	{ "kangaroo.notify.interval","14400" },
	{ "kangaroo.notify.program", "/bin/mail -s \"[Kangaroo] problem\"" },
	{ "kangaroo.nearest.host", "localhost" },
	{ "kangaroo.debug", 0 },
	{ 0, 0 },
};

static struct config_table * kangaroo_config_table = 0;

static int kangaroo_config_init()
{
	struct entry *e;
	const char *filename;
	int save_errno;
	int success;

	if(kangaroo_config_table) return 1;

	kangaroo_config_table = config_table_create();
	if(!kangaroo_config_table) goto failure;

	/* First set most values according to static defaults */

	for(e=defaults;e->key;e++) {
		success = config_table_set(kangaroo_config_table,e->key,e->value);
		if(!success) goto failure;
	}

	/* Load everything else from the config file */

	filename = config_table_get(kangaroo_config_table,"kangaroo.config");
	if(!filename) goto failure;

	if(!config_table_load(kangaroo_config_table,filename)) {
		debug("warning: couldn't load %s (%s) continuing with defaults\n",filename,strerror(errno));
	}

	return 1;

	failure:
	save_errno = errno;
	if(kangaroo_config_table) {
		config_table_delete(kangaroo_config_table);
	}
	kangaroo_config_table = 0;
	errno = save_errno;
	return 0;

}

int kangaroo_config_load( const char *filename )
{
	if(!kangaroo_config_init()) return 0;
	return config_table_load(kangaroo_config_table,filename);
}

int kangaroo_config_save( const char *filename )
{
	if(!kangaroo_config_init()) return 0;
	return config_table_save(kangaroo_config_table,filename);
}

int kangaroo_config_set( const char *key, const char *value )
{
	if(!kangaroo_config_init()) return 0;
	return config_table_set(kangaroo_config_table,key,value);
}

char *kangaroo_config_get( const char *key )
{
	if(!kangaroo_config_init()) return 0;
	return config_table_get(kangaroo_config_table,key);
}

int kangaroo_config_set_int( const char *key, int value )
{
	if(!kangaroo_config_init()) return 0;
	return config_table_set_int(kangaroo_config_table,key,value);
}

int kangaroo_config_get_int( const char *key )
{
	if(!kangaroo_config_init()) return 0;
	return config_table_get_int(kangaroo_config_table,key);
}


