/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/


#include "kangaroo_error.h"

#include <errno.h>

int kangaroo_error_to_unix( int error )
{
	switch(error) {
		case KANGAROO_ERROR_NO_FILE:
			return ENOENT;
		case KANGAROO_ERROR_NO_MEMORY:
			return ENOMEM;
		case KANGAROO_ERROR_NO_DISK:
			return ENOSPC;
		case KANGAROO_ERROR_PROTOCOL:
			return EPROTO;
		case KANGAROO_ERROR_AUTHENTICATION:
		case KANGAROO_ERROR_AUTHORIZATION:
			return EACCES;
		case KANGAROO_ERROR_NOT_IMPLEMENTED:
			return ENOSYS;
		case KANGAROO_ERROR_CONNECTION_RESET:
			return ECONNRESET;
		case KANGAROO_ERROR_CONNECTION_REFUSED:
			return ECONNREFUSED;
		case KANGAROO_ERROR_HOST_UNREACHABLE:
			return EHOSTUNREACH;
		case KANGAROO_ERROR_INVALID:
		default:
			return EINVAL;
			break;
	}
}

int kangaroo_error_from_unix( int error )
{
	switch(error) {
		case ENOENT:
			return KANGAROO_ERROR_NO_FILE;
		case ENOMEM:
			return KANGAROO_ERROR_NO_MEMORY;
		case ENOSPC:
			return KANGAROO_ERROR_NO_DISK;
		case EPROTO:
			return KANGAROO_ERROR_PROTOCOL;
		case EPERM:
		case EACCES:
			return KANGAROO_ERROR_AUTHORIZATION;
		case EPIPE:
                case ECONNRESET:
                        return KANGAROO_ERROR_CONNECTION_RESET;
		case ECONNREFUSED:
			return KANGAROO_ERROR_CONNECTION_REFUSED;
		case EHOSTUNREACH:
			return KANGAROO_ERROR_HOST_UNREACHABLE;
		case ENOSYS:
			return KANGAROO_ERROR_NOT_IMPLEMENTED;
		default:
			return KANGAROO_ERROR_INVALID;
	}
}

int kangaroo_error_is_final( int error )
{
	switch(error) {
		case KANGAROO_ERROR_NO_FILE:
		case KANGAROO_ERROR_AUTHORIZATION:
		case KANGAROO_ERROR_INVALID:
			return 1;
		default:
			return 0;
	}
}

