

#include "kangaroo_history.h"
#include "histogram.h"

#include <pthread.h>

static struct histogram *incoming_minute=0;
static struct histogram *incoming_hour=0;
static struct histogram *incoming_day=0;

static struct histogram *outgoing_minute=0;
static struct histogram *outgoing_hour=0;
static struct histogram *outgoing_day=0;

static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;

static int kangaroo_history_init()
{
	if(!incoming_minute) {
		incoming_minute = histogram_create(60,1);
		if(!incoming_minute) return 0;
	}
	if(!incoming_hour) {
		incoming_hour = histogram_create(60,60);
		if(!incoming_hour) return 0;
	}
	if(!incoming_day) {
		incoming_day = histogram_create(24,2600);
		if(!incoming_day) return 0;
	}
	if(!outgoing_minute) {
		outgoing_minute = histogram_create(60,1);
		if(!outgoing_minute) return 0;
	}
	if(!outgoing_hour) {
		outgoing_hour = histogram_create(60,60);
		if(!outgoing_hour) return 0;
	}
	if(!outgoing_day) {
		outgoing_day = histogram_create(24,3600);
		if(!outgoing_day) return 0;
	}
	return 1;
}

int kangaroo_history_outgoing_add( UINT64_T bytes )
{
	int success;
	pthread_mutex_lock(&mutex);
	success = kangaroo_history_init();
	if(success) {
		histogram_delta(outgoing_minute,0,bytes);
		histogram_delta(outgoing_hour,0,bytes);
		histogram_delta(outgoing_day,0,bytes);
	}
	pthread_mutex_unlock(&mutex);
	return 1;
}

int kangaroo_history_incoming_add( UINT64_T bytes )
{
	int success;
	pthread_mutex_lock(&mutex);
	success = kangaroo_history_init();
	if(success) {
		histogram_delta(incoming_minute,0,bytes);
		histogram_delta(incoming_hour,0,bytes);
		histogram_delta(incoming_day,0,bytes);
	}
	pthread_mutex_unlock(&mutex);
	return success;
}

int kangaroo_history_outgoing_get( UINT64_T *minute, UINT64_T *hour, UINT64_T *day )
{
	int success;
	*minute = *hour = *day = 0;
	pthread_mutex_lock(&mutex);
	success = kangaroo_history_init();
	if(success) {
		*minute = histogram_total(outgoing_minute);
		*hour = histogram_total(outgoing_hour);
		*day = histogram_total(outgoing_day);
	}
	pthread_mutex_unlock(&mutex);
	return success;
}

int kangaroo_history_incoming_get( UINT64_T *minute, UINT64_T *hour, UINT64_T *day )
{
	int success;
	*minute = *hour = *day = 0;
	pthread_mutex_lock(&mutex);
	success = kangaroo_history_init();
	if(success) {
		*minute = histogram_total(incoming_minute);
		*hour = histogram_total(incoming_hour);
		*day = histogram_total(incoming_day);
	}
	pthread_mutex_unlock(&mutex);
	return 1;
}


