/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/


#include "kangaroo_message.h"

#include <stdlib.h>
#include <string.h>

struct mtable {
	int type;
	char *name;
	char *format;
	int fields;
	int has_data;
};

/* Warning: The order of this table must match kangaroo_message.h */

static struct mtable table[] = {
	{ KANGAROO_CREATE,     "create",      "%s %s %5$d",     3, 0 },
	{ KANGAROO_DELETE,     "delete",      "%s %s",          2, 0 },
	{ KANGAROO_RENAME,     "rename",      "%s %s %6$s",     3, 0 },
	{ KANGAROO_TRUNCATE,   "truncate",    "%s %s %d",       3, 0 },
	{ KANGAROO_PUT,        "put",         "%s %s %d %d",    4, 1 },
	{ KANGAROO_GET,        "get",         "%s %s %d %d",    4, 0 },
	{ KANGAROO_PUSH,       "push",        "%s %s",          2, 0 },
	{ KANGAROO_COMMIT,     "commit",      "",               0, 0 },
	{ KANGAROO_ROUTE,      "route",       "%s",             1, 0 },
	{ KANGAROO_UNQUEUE,    "unqueue",     "%s %s",          2, 0 },
	{ KANGAROO_PAUSE,      "pause",       "%s %s",          2, 0 },
	{ KANGAROO_RESUME,     "resume",      "%s %s",          2, 0 },
	{ KANGAROO_STATUS,     "status",      "",               0, 0 },
	{ 0,                   0,             0,                0, 0 },
};


struct kangaroo_message * kangaroo_message_read( FILE *stream, int with_data )
{
	struct kangaroo_message *m;
	char line[KANGAROO_LINE_MAX];
	char command[KANGAROO_LINE_MAX];
	char *rest;
	int fields;
	int result;
	int i;

	if(!fgets(line,KANGAROO_LINE_MAX,stream)) return 0;

	if(line[0]==4) return 0;

	fields = sscanf(line,"%s",command);
	if(fields!=1) return 0;

	rest = &line[strlen(command)+1];

	m = malloc(sizeof(*m));
	if(!m) return 0;

	memset(m,0,sizeof(*m));

	for( i=0; table[i].name; i++ ) {
		if(!strcmp(table[i].name,command)) {
			m->type = table[i].type;
			if(table[i].format[0]) {
				fields = sscanf(rest,table[i].format,m->addr,m->path,&m->offset,&m->length,&m->mode,m->newpath);
			} else {
				fields = 0;
			}
			if(fields!=table[i].fields) break;

			if(with_data && table[i].has_data) {

				m->data = malloc(m->length);
				if(!m->data) break;

				result = fread(m->data,1,m->length,stream);
				if(result!=m->length) {
					kangaroo_message_delete( m );
					return 0;
				}
			}
			return m;
		}
	}

	kangaroo_message_delete(m);
	return 0;
}

int kangaroo_message_write( FILE *stream, struct kangaroo_message *m, int with_data )
{
	int wcount;
	int result;
	struct mtable *t;

	t = &table[m->type];
	
	wcount = 0;
	result = 0;
	wcount += fprintf(stream,"%s ",t->name);
	wcount += fprintf(stream,t->format,m->addr,m->path,m->offset,m->length,m->mode,m->newpath);
	wcount += fprintf(stream,"\n");
	if( with_data && m->data && t->has_data ) {
		result = fwrite( m->data, 1, m->length, stream );
		if(result!=m->length) return 0;
	}

  debug ("Has written wcount %d, more %d\n", wcount, result);
  wcount += result;
  return wcount;
	
}

char * kangaroo_message_name( struct kangaroo_message *m )
{
	return table[m->type].name;
}

void kangaroo_message_release_data( struct kangaroo_message *m )
{
	if(m->data) {
		free(m->data);
		m->data = 0;
	}
}

void kangaroo_message_delete( struct kangaroo_message *m )
{
	if(m) {
		if(m->data) free(m->data);
		free(m);
	}
}

int kangaroo_space_required (struct kangaroo_message *m, int with_data)
{
  int count;
	char tbuf[1024];
  struct mtable *t;

  t = &table[m->type];
	
  count = 0;
  count += sprintf(tbuf,"%s ",t->name);
	count += sprintf(tbuf,"%s %s %s %d %d %d %s",t->format,m->addr,m->path,m->offset,m->length,m->mode,m->newpath);
	count++;
	if( with_data && m->data && t->has_data ) {
		count+= m->length;
	}

  return count;
}

int kangaroo_has_data(struct kangaroo_message *m)
{
  struct mtable *t;
  t = &table[m->type];

  return t->has_data;
}

