/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/


#ifndef KANGAROO_MESSAGE_H
#define KANGAROO_MESSAGE_H

#include <stdio.h>
#include <errno.h>

#include "kangaroo_types.h"
#include "debug.h"

/* Warning: The order of this table must match kangaroo_message.c */

enum kangaroo_message_type {
	KANGAROO_CREATE,
	KANGAROO_DELETE,
	KANGAROO_RENAME,
	KANGAROO_TRUNCATE,
	KANGAROO_PUT,
	KANGAROO_GET,
	KANGAROO_PUSH,
	KANGAROO_COMMIT,
	KANGAROO_ROUTE,
	KANGAROO_UNQUEUE,
	KANGAROO_PAUSE,
	KANGAROO_RESUME,
	KANGAROO_STATUS,
};

struct kangaroo_message {
	enum kangaroo_message_type type;
	char addr[KANGAROO_ADDR_MAX];
	char path[KANGAROO_PATH_MAX];
	char tag[KANGAROO_LINE_MAX];
	int offset;
	int length;
	int mode;
	char newpath[KANGAROO_PATH_MAX];
	char *data;
};

struct kangaroo_message * kangaroo_message_read( FILE *stream, int with_data );
int kangaroo_message_write( FILE *stream, struct kangaroo_message *m, int with_data );
void kangaroo_message_release_data( struct kangaroo_message *m );
void kangaroo_message_delete( struct kangaroo_message *m );
char *kangaroo_message_name( struct kangaroo_message *m );
int kangaroo_space_required (struct kangaroo_message *m, int with_data);
int kangaroo_has_data(struct kangaroo_message *m);

#endif









