/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/


#include "kangaroo_receive.h"
#include "kangaroo_execute.h"
#include "kangaroo_error.h"
#include "kangaroo_types.h"
#include "kangaroo_message.h"

#include "client_cache.h"

#include <stdlib.h>
#include <errno.h>

void kangaroo_receive( FILE *rstream, FILE *wstream )
{
	struct client_cache *cache;
	struct kangaroo_message *msg;
	int result;

	cache = client_cache_create();
	if(!cache) return;

	while(1) {
		msg = kangaroo_message_read(rstream,1);
		if(!msg) break;

		switch(msg->type) {

			/* These calls return nothing. */

			case KANGAROO_CREATE:
			case KANGAROO_DELETE:
			case KANGAROO_RENAME:
			case KANGAROO_TRUNCATE:
			case KANGAROO_PUT:
				result = kangaroo_execute_put(msg,cache);
				if(result<0) break;
				break;

			/* These calls return an int, followed by data */

			case KANGAROO_GET:
				result = kangaroo_execute_get(msg,cache);
				if(result<0) result = kangaroo_error_from_unix(errno);
				fprintf(wstream,"%d\n",result);
				if(result>0) fwrite(msg->data,1,result,wstream);
				fflush(wstream);
				kangaroo_message_delete(msg);
				break;
			case KANGAROO_ROUTE:
				result = kangaroo_execute_route(msg,cache);
				if(result<0) result = kangaroo_error_from_unix(errno);
				fprintf(wstream,"%d\n",result);
				if(result>0) fwrite(msg->data,1,result,wstream);
				fflush(wstream);
				kangaroo_message_delete(msg);
				break;
			case KANGAROO_STATUS:
				result = kangaroo_execute_status(msg,cache);
				if(result<0) result = kangaroo_error_from_unix(errno);
				fprintf(wstream,"%d\n",result);
				if(result>0) fwrite(msg->data,1,result,wstream);
				fflush(wstream);
				kangaroo_message_delete(msg);
				break;

			/* These calls return an int */

			case KANGAROO_PUSH:
				result = kangaroo_execute_push(msg,cache);
				if(result<0) result = kangaroo_error_from_unix(errno);
				fprintf(wstream,"%d\n",result);
				fflush(wstream);
				kangaroo_message_delete(msg);
				break;
			case KANGAROO_COMMIT:
				result = kangaroo_execute_commit(msg,cache);
				if(result<0) result = kangaroo_error_from_unix(errno);
				fprintf(wstream,"%d\n",result);
				fflush(wstream);
				kangaroo_message_delete(msg);
				break;
			case KANGAROO_UNQUEUE:
				result = kangaroo_execute_unqueue(msg,cache);
				if(result<0) result = kangaroo_error_from_unix(errno);
				fprintf(wstream,"%d\n",result);
				fflush(wstream);
				kangaroo_message_delete(msg);
				break;
			case KANGAROO_PAUSE:
				result = kangaroo_execute_pause(msg,cache);
				if(result<0) result = kangaroo_error_from_unix(errno);
				fprintf(wstream,"%d\n",result);
				fflush(wstream);
				kangaroo_message_delete(msg);
				break;
			case KANGAROO_RESUME:
				result = kangaroo_execute_resume(msg,cache);
				if(result<0) result = kangaroo_error_from_unix(errno);
				fprintf(wstream,"%d\n",result);
				fflush(wstream);
				kangaroo_message_delete(msg);
				break;

			default:
				fprintf(wstream,"%d\n",KANGAROO_ERROR_NOT_IMPLEMENTED);
				fflush(wstream);
				break;
		}
	}

	client_cache_delete(cache);
}
