/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/


#include "kangaroo_summary.h"
#include "message_list.h"
#include "hash_table_to_list.h"

#include <stdlib.h>
#include <stdio.h>
#include <sys/vfs.h>

static int cleanup( struct hash_table *table, char *key, void *value, void *arg )
{
	hash_table_remove(table,key);
	free(value);
	return 1;
}

static int summarize( struct kangaroo_message *m, void *arg, enum kangaroo_summary_type type )
{
	struct hash_table *table = (struct hash_table *)arg;
	struct kangaroo_summary *s;
	char key[KANGAROO_PATH_MAX];

	switch(type) {
		case KANGAROO_SUMMARY_BY_HOST:
			strcpy(key,m->addr);
			break;
		case KANGAROO_SUMMARY_BY_FILE:
			sprintf(key,"%s:%s",m->addr,m->path);
			break;
		default:
			return 0;
	}

	s = hash_table_lookup( table, key );
	if(!s) {
		s = malloc(sizeof(*s));
		if(!s) return 0;

		strcpy(s->host,m->addr);
		if(type==KANGAROO_SUMMARY_BY_FILE) {
			strcpy(s->path,m->path);
		} else {
			strcpy(s->path,"*");
		}

		s->ops = 0;
		s->bytes = 0;

		if(!hash_table_insert( table, key, s, 0)) {
			free(s);
			return 0;
		}
	}

	s->ops++;
	s->bytes+=m->length;

	return 1;
}

static int summarize_by_host( struct kangaroo_message *m, void *arg )
{
	return summarize(m,arg,KANGAROO_SUMMARY_BY_HOST);
}

static int summarize_by_file( struct kangaroo_message *m, void *arg )
{
	return summarize(m,arg,KANGAROO_SUMMARY_BY_FILE);
}

struct list * kangaroo_summary_create( struct message_list *m, enum kangaroo_summary_type type )
{
	struct hash_table *table;
	struct list *list;
	int keepgoing;

	table = hash_table_create(127,hash_string);
	if(!table) return 0;

	if( type==KANGAROO_SUMMARY_BY_HOST ) {
		keepgoing = message_list_iterate( m, summarize_by_host, table );
	} else {
		keepgoing = message_list_iterate( m, summarize_by_file, table );
	}

	list = hash_table_to_list(table);
	if(list) {
		hash_table_delete( table );
		return list;
	} else {
		hash_table_iterate( table, cleanup, 0 );
		hash_table_delete( table );
		return 0;
	}
}

static int sum_up( struct kangaroo_message *m, void *arg )
{
	long long *total = arg;
	*total += m->length;
	return 1;
}

int kangaroo_summary_total( struct message_list *m, long long *kb_used )
{
	*kb_used = 0;
	message_list_iterate(m,sum_up,kb_used);
	return 0;
}


