/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/
/*
 Part of multi route version of kangaroo
 Modified by Gogul Balakrishnan and Rajesh Rajamani
*/ 



#include <pthread.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "kangaroo_types.h"
#include "kangaroo_config.h"
#include "kangaroo_client.h"
#include "kangaroo_execute.h"
#include "kangaroo_history.h"

#include "message_list.h"
#include "spool_dir.h"
#include "routing_table.h"
#include "debug.h"
#include "mover.h"
#include "macros.h"
#include "timestamp.h"
#include "error_state.h"
#include "dep_graph.h"
#include "list.h"

//static char last_hop[KANGAROO_HOST_MAX] = { 0 };	//unnecessary for our case
//static int messages_out = 0;	//unnecessary
extern pthread_mutex_t mover_receiver_lock;
static int pause_mode = 0;	//keep this global for this mover
struct hop_info {
								char *nxthop;
								struct kangaroo_server *server;
								time_t last_commit;
							};

typedef struct	{
									int mover_id;
									int index;
									struct hop_info *hinfo;
									struct dep_graph *dg;
								} mover_info;

extern struct routing_table *kangaroo_routing_table;
extern struct message_list *kangaroo_message_list;

void* mover_info_initialize (int mover_id, int threadindex, void *arg)	{
	mover_info *minfo = calloc(1, sizeof(mover_info));

	minfo->mover_id = mover_id;
	minfo->index = threadindex;
	if (mover_id == 0)	{
		minfo->dg = arg;	
	}
	else	{
		minfo->hinfo = arg;
	}
	return minfo;
}

void mover_pause()
{
	pause_mode = 1;
	error_state_clear();
}

void mover_resume()
{
	pause_mode = 0;
	error_state_clear();
}

int mover_is_paused()
{
	return pause_mode;
}

static void mover_abort(mover_info *minfo)
{
	debug ("mover: abort calls rollback\n");
	dep_graph_rollback(minfo->dg, minfo->mover_id, 0);
	if(minfo->hinfo->server) {
		kangaroo_server_close(minfo->hinfo->server);
		minfo->hinfo->server = 0;
	}
}

/*
static int mover_message_push_all(void *i, const void *arg) {
	int result;
	struct kangaroo_message *msg;
	struct dep_graph_node *dnode =i;
	struct kangaroo_server *server =(struct kangaroo_server*) arg;

	msg = dnode->m;
	if (msg == NULL)	
		return 1;

	result = kangaroo_push (server,	msg->addr, msg->path);
	//if (result <0)	{
		//return 0;
	//}

	//return 1;	//This will send pushes for every msg in the list
	return 0;
}
*/


static int mover_push(mover_info *minfo, struct list *push_list)	{
	struct dep_graph_node *dnode;
	struct kangaroo_message *msg;
	struct kangaroo_server *server;
	char *addr;
	char *hop;
	int result;

	if (minfo == NULL)	{
		return 0;
	}
	if (minfo->dg == NULL)	{
		return 0;
	}

	hop = minfo->hinfo->nxthop;
	addr = dep_graph_destination(minfo->dg);

	//result = routing_table_get_hop (kangaroo_routing_table, addr, hop);
	//if (!result)	{
	if (!hop)	{
		strcpy (hop, "unknown");
		return 0;
	}

	if (hop[0] == '@')	{
		debug ("mover: successfully pushed along %d\n", minfo->mover_id);
		return 1;
	}

	server = minfo->hinfo->server;
	if(!server)	{
		debug ("mover: opening connection to %s\n", hop);
		server = kangaroo_server_open(hop,0); 
		if(!server) return 0;
		minfo->hinfo->server = server;
	}

	//result = list_iterate(push_list,mover_message_push_all,server);

	if ((dnode = list_peek_tail(push_list)) != 0)	{
		msg = dnode->m;
		if (msg != NULL)	{
			result = kangaroo_push (server, msg->addr, msg->path);
			if ((result <0) && (server))	{
				kangaroo_server_close(server);
				minfo->hinfo->server = 0;
				return 0;
			}
		}	
	}

	if ((result < 0) && (server))	{
		kangaroo_server_close(server); 
		minfo->hinfo->server = 0; 
		return 0;   
	}	

	debug ("mover: successfully pushed along %d\n", minfo->mover_id);
	return 1;
}

static int mover_send(mover_info *minfo, struct kangaroo_message *incore )
{
	struct kangaroo_server *server;
	char *hop;
	int save_errno;
	int result;
	
	if (incore == NULL)	{
		return 0;
	}	
	if (minfo == NULL)	{
		debug("mover: minfo is NULL!!! \n");
		return 0;
	}

	server = minfo->hinfo->server;
	hop = minfo->hinfo->nxthop;
	
	//result = routing_table_get_hop (kangaroo_routing_table, incore->addr, hop);
	//if (!result)	{
	if (!hop)	{	
		strcpy (hop, "unknown");
		goto failure;
	}

	if (hop[0] == '@')	{
		result = kangaroo_execute_put_local(incore,0);
		if (result >= 0)	{
			goto success;
		}
		else	{
			goto failure;
		}
	}
	
	if (!server)	{
		debug ("mover: opening connection to %s\n", hop);
		server = kangaroo_server_open(hop,0); 
		if(!server) goto failure;
		minfo->hinfo->server = server;
	}

	result = kangaroo_message_send(server, incore);
	if (!result)	goto failure;

	success:
	kangaroo_history_outgoing_add(incore->length);
	debug("mover: sent packet successfully\n");
	return 1;

	failure:
	save_errno = errno;
	if(server)
		kangaroo_server_close(server);
	minfo->hinfo->server = 0;
	debug ("mover: couldn't send to %s: %s\n",hop,strerror(errno));
	errno = save_errno; 
	return 0;
}

void* mover_multi_thread(void *arg)	{

	int delay, kangaroo_default_delay;
	int last_time, real_rollback; 
	mover_info *input;
	pthread_t thread_ids[DEP_GRAPH_MAX_MOVERS];
	memset (thread_ids, -1, sizeof(thread_ids));

	input = arg;
	last_time = 0;
	real_rollback = 0;
	kangaroo_default_delay = kangaroo_config_get_int("kangaroo.mover.timeout.min");   

	if (input->mover_id == 0)	{
		/* This is the main thread, has to spawn off other mover threads - one per each
			 available next link.  After that each thread functions in a similar fashion */

			 int i;
			 struct routing_table *plist;
			 struct routing_table_iterator *piterator;
			 char *pnxthop;
			 char *dest = dep_graph_destination(input->dg);
			 int numhops = 0;
			 int nmovers = 0;

			 //memset(thread_ids, 0, sizeof(thread_ids));
			 
			 gethops:
			 plist = routing_table_get_all_hops (kangaroo_routing_table, dest);
			 numhops = routing_table_num_routes (plist);
			 piterator = routing_table_iterator_init(plist);
			 if (numhops == 0)	{
					debug("mover: no routes available to %s.\n",dest);
					sleep(5);
					goto gethops;
			 }

			 /* register numhops with dep_graph */
			 registeragain:
			 nmovers = dep_graph_number_next_hops(input->dg, numhops);
			 if (nmovers == 0)	{
					debug("mover: dep_graph unable to register any mover ... will try again.\n");
					sleep(5);
					goto registeragain;
			 }

			 for (i=0; i<nmovers; i++)	{
					struct hop_info *hinfo;
					int toinit = 1;

					pnxthop = routing_table_iterator_nexthop(piterator);
					if (pnxthop == 0)	{
						toinit = 1;
						debug ("mover: nxthop is NULL!!! ... no mover initialized.\n");
					}

					if (toinit)	{
						hinfo = calloc(1, sizeof(struct hop_info));
						hinfo->nxthop = strdup(pnxthop);
					}	
					if ((i == 0) && (toinit))	{
						input->hinfo = hinfo;
					}
					else	if (toinit) {
						mover_info *thread_input = mover_info_initialize (i,0,hinfo); 	
						thread_input->dg = input->dg;

						while (pthread_create(&thread_ids[i],0,mover_multi_thread,thread_input) != 0)	{
							debug("incoming: couldn't create new thread because %s\n",strerror(errno));
							debug("incoming: try again in %d seconds\n",delay);
							timestamp_sleep(delay*1000000); 
							delay = MIN(delay*2,kangaroo_config_get_int("kangaroo.incoming.timeout.max"));
						}
					}
			 	} 
			 routing_table_iterator_destroy(piterator);
	}/*if mover id is zero */

	/* From here all threads have similar functionality */
	debug("mover: thread %d reaches here\n",input->mover_id);
	while (1)	{
		int save_errno;
		struct kangaroo_message *m;
		int getnextfailed = 0;	/* If u r unable to get packets for 3 times, push whaterver
															 has been sent */

		m = 0;
		//delay = kangaroo_config_get_int("kangaroo.mover.timeout.min");    
		delay = kangaroo_default_delay;

		if (pause_mode)	{
			debug("mover: paused at user request.\n");
			while(pause_mode) {
			     sleep(1);
			}
			debug("mover: resumed at user request.\n");
		}

		while ( ((m = dep_graph_get_next(input->dg, input->mover_id)) == 0)
						&& (getnextfailed < 3) && (last_time == 0) )	{
			timestamp_sleep(delay*1000000);
			delay = MIN(delay*2,kangaroo_config_get_int("kangaroo.mover.timeout.min"));
			getnextfailed++;
		}

		if (pause_mode)	{
			errno = EINTR;
			goto failure;
		}

		/* At this point, either there is a message to send, or all messages have to be
			 pushed, or dep_graph is empty */
		if (m)	{
				// send this message
				if (mover_send(input, m))	{
					last_time = 0;
					real_rollback = 1;
				}
				else	{
					debug("mover: unable to send packet ... rollback called %d\n", real_rollback);
					timestamp_sleep(delay*1000000);
					dep_graph_rollback(input->dg, input->mover_id, real_rollback);
					goto failure;
				}
		}
		else	if (getnextfailed >= 3)	{
			/* get list of messages to be pushed and push */
			struct list *push_list;
			push_list = dep_graph_what_to_push (input->dg, input->mover_id);
			debug("mover: dep_graph doesn't give next packet to mover %d\n",input->mover_id);
			if (push_list)	{
				if (!mover_push(input, push_list))	{
					debug("push failed %d\n", real_rollback);
					dep_graph_rollback(input->dg, input->mover_id, real_rollback);
					timestamp_sleep(delay*1000000);
					last_time = 1;
					goto failure;
				}
				debug("mover: push was successfull\n");
				dep_graph_pushed(input->dg, input->mover_id);	//successfully pushed
				real_rollback = 0;
			}
			last_time = 1;
		}
		else	if (last_time == 1)	{
			/* pushed last time and still no messages!!! Ask if dep_graph is empty.  If empty,
				 mover can quit.  While quitting, if mover_id is zero, wait for other threads to
				 join and then decrement kangaroo_execute_mover_count_decrement */
				 last_time = 0;

				 if (dep_graph_is_empty(input->dg))	{
						if (input->mover_id == 0)	{
							int i;
							for (i=0; i<DEP_GRAPH_MAX_MOVERS; i++)	{
								if (thread_ids[i] != -1)	{
									pthread_join(thread_ids[i], 0);
									debug("mover: thread with id %d has joined\n", i);
								}
							}
							pthread_mutex_lock(&mover_receiver_lock);
							kangaroo_execute_mover_count_decrement(input->index);
							debug ("mover: deleting dep_graph\n");
							pthread_cond_broadcast(&input->dg->pushcond);
							dep_graph_delete (input->dg);
							pthread_mutex_unlock(&mover_receiver_lock);
							debug ("mover: dep_graph deleted\n");
						}

						if (input->hinfo->server)
							kangaroo_server_close(input->hinfo->server);
						free(input->hinfo->nxthop);
						free(input->hinfo);
						free(input);
						debug("mover: mover with id %d exits\n",input->mover_id);
						pthread_exit(0);
				}
		}

		continue;

	failure:
  save_errno = errno;
	if (m)	{
		 error_state_set(m->addr,m->path,"couldn't send '%s %s' to %s because %s",kangaroo_message_name(m),m->path,m->addr,strerror(save_errno));
	}
	else	{
		/*
		char host[KANGAROO_HOST_MAX]; 
		char *tmp;
		if (!input->hinfo->nxthop)	{
			strcpy(host,"unknown"); 
		}  
		else	{
			strcpy (host, input->hinfo->nxthop);
		}
		 tmp = strerror(save_errno);
		 error_state_set(host,"","couldn't send 'push' to %s because %s",host,tmp);
		*/ 
	}
	debug("mover: try again in %d seconds\n",delay);
	//mover_abort(input); 
	timestamp_sleep(delay*1000000);  
	delay = MIN(delay*2,kangaroo_config_get_int("kangaroo.mover.timeout.max"));  
  }//end while(1)	

	/* Before exiting, delete input->hinfo->nxthop, input->hinfo, input->dg (call dep_graph fn to do this), input */
	return NULL;
}
