/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/


#ifndef ROUTING_TABLE_H
#define ROUTING_TABLE_H

#ifdef _cplusplus
extern "C" {
#endif

/*
Load a routing table from a file.
On success, return an object.
On failure, return 0 with errno set.
*/

struct routing_table * routing_table_load( const char *filename );

/*
Delete an existing routing table.
*/

void routing_table_delete( struct routing_table *r );

/*
Search a given routing table for a rule governing 'dest'.
On success, return 1 and fill 'hop' with the next hop.
On failure, return 0 with errno set.
Special values for 'hop' may be:
    STORE   Deliver data to this filesystem.
    HOLD    Hold data in this spool.  (for now.)
*/

int routing_table_get_hop( struct routing_table *r, const char *dest, char *hop );
struct routing_table* routing_table_get_all_hops (struct routing_table *r, const char *dest);

/* To get number of routes to destination - returns -1 on error, 0 if
   this is the destination, else number of distinct routes */
int routing_table_num_routes (const struct routing_table *plist);

/* call cleanup when u no longer need routing_table* returned by routing_table_get_all_hops */
void routing_table_cleanup (struct routing_table *plist);

struct routing_table_iterator* routing_table_iterator_init (struct routing_table *plist);
void routing_table_iterator_destroy (struct routing_table_iterator *piterator);

/* To iterate through the various entries in the list */
char* routing_table_iterator_nexthop (struct routing_table_iterator *piterator);

#ifdef _cplusplus
}
#endif

#endif
