/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/


#include "stringtools.h"

#include <string.h>
#include <stdio.h>
#include <math.h>

#define STRINGTOOLS_BUFFER_SIZE 256
#define METRIC_POWER_COUNT 6

int pattern_match( const char *pattern, const char *text )
{
        char *w;
        int headlen, taillen;

        w = strchr( pattern, '*' );
        if(!w) return !strcmp(pattern,text);

        headlen = w-pattern;
        taillen = strlen(pattern)-headlen-1;

        return !strncmp(pattern,text,headlen) && !strcmp(&pattern[headlen+1],&text[strlen(text)-taillen]);
}

char * strfront( const char *str, int max )
{
	static char buffer[STRINGTOOLS_BUFFER_SIZE];
	int length;

	length = strlen(str);
	if(length<max) {
		strcpy(buffer,str);
	} else {
		strncpy(buffer,str,max);
		buffer[max] = 0;
	}
	return buffer;
}

const char * strback( const char *str, int max )
{
	int length;

	length = strlen(str);
	if(length<max) {
		return str;
	} else {
		return &str[length-max];
	}
}

void chomp( char *buffer )
{
        int pos;
        char *c;

        pos = strlen(buffer);
        c = &buffer[pos];

        while(1) {
                c--;
                if( (*c=='\n') || (*c=='\r') ) {
                        *c=0;
                } else {
                        break;
                }
        }
}

void metric_units( int invalue, int power_needed, char *buffer )
{
	static char *suffix[METRIC_POWER_COUNT] =
		{ " ", "K", "M", "G", "T", "P" };

	double value=invalue;
	int power=0;

	if(power_needed==-1) {
		while( (value>=1024.0) && (power<(METRIC_POWER_COUNT-1)) ) {
			value = value / 1024.0;
			power++;
		}
	} else {
		power = power_needed;
		value = value / (pow(2,10*power));
	}

	sprintf( buffer, "%.1f %s", value, suffix[power] );
}





