/*
Kangaroo is Copyright (C) 2001 Douglas Thain.
Permission is granted to use the source code for research and study purposes.
Other uses must be cleared with the copyright holder.
*/


#include "username.h"

#include <pthread.h>
#include <pwd.h>
#include <unistd.h>
#include <sys/types.h>

static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;

int username_is_super()
{
	return !getuid();
}

int username_get( char *name )
{
	struct passwd *p;

	pthread_mutex_lock(&mutex);

	p = getpwuid(getuid());
	if(p) {
		strcpy(name,p->pw_name);
		pthread_mutex_unlock(&mutex);
		return 1;
	} else {
		pthread_mutex_unlock(&mutex);
		return 0;
	}
}

int username_set( const char *name )
{
	struct passwd *p;
	int result;
	uid_t uid;
	gid_t gid;

	pthread_mutex_lock(&mutex);

	p = getpwnam(name);
	if(!p) {
		pthread_mutex_unlock(&mutex);
		return 0;
	}

	uid = p->pw_uid;
	gid = p->pw_gid;

	pthread_mutex_unlock(&mutex);

	result = seteuid(0);
	if(result<0) return 0;

	setuid(uid);
	setgid(gid);

	return 1;
}
